/*************************************************************************//**
* \class	Celula
* \brief 	Definição do elemento celula
* \author	Cyro Albuquerque Neto
* \date		Criação: 18/08/2008
* \date		Revisão: 03/06/2009
*
* Define o elemento celula.
*****************************************************************************/

#ifndef CELULAH
#define CELULAH

// INCLUDES DO SISTEMA
#include <cmath>

// INCLUDES LOCAIS
#include "definicao.h"
#include "ponto.h"
#include "camada.h"
#include "compartimento.h"

using namespace std;

class Celula;
class Face;

class Celula {
public:

// CONSTRUTORES E DESTRUTORES

	/** Construtor
	 *
	 * \param nDim			Número de dimensões [1-3] 
	 * \param sistCoord		Sistema de coordenadas
	 * \param cam			Vetor com endereço das camadas
	 * \param jCel			Posição da célula
	 * \param cTec			Endereço do compartimento tecidual
	 * \param tamExt		Tamanho externo
	 */
	Celula( IntPos				nDim, 
			SistCoord			sistCoord, 
			vector<Camada*>&	cam, 
			VetInt&				jCel, 
			CompTecido*			cTec, 
			Ponto<double>		tamExt);

	/** Destrutor
	 */
	~Celula() {}

// ACESSO
	
	/** Ponto no centro da célula
	 */
	Ponto<double> Centro() const { return centro; }
	
	/** Volume da célula
	 */
	double Volume() const { return volume; }
	
	/** Atribui valor para a temperatura da célula
	 * \param temperatura Temperatura [oC]
	 */
	void SetTemperaturaAnt(double valor) { temperaturaAnt=valor; }
	void SetTemperatura(double valor) { temperatura=valor; }

	/** Soma valor na temperatura da célula
	 * \param temperatura Temperatura [oC]
	 */
	void SomaTemperatura(double valor) { temperatura+=valor; }

	/** Obtêm valor da temperatura da célula
	 * \return Temperatura da célula [oC]
	 * */
	double Temperatura() const { return temperatura; }
	double TemperaturaAnt() const { return temperaturaAnt; }
	
	/** Atribui valor para o tamanho da célula
	 * \param val1 Tamanho da célula na direção 1 [m]
	 * \param val2 Tamanho da célula na direção 2 [m ou rad]
	 * \param val3 Tamanho da célula na direção 3 [m]
	 */
	void SetTamanho(double val1, 
					double val2, 
					double val3) { 
		tamanho.SetPonto(val1,val2,val3); 
	}

	/** Obtêm tamanho da célula
	 * \return tamanho da célula [m; m ou rad; m]
	 */
	Ponto<double> Tamanho() const { return tamanho; }

	//void SetTamanhoCoord(IntPos k, double val) 
	//	{ tamanho.SetPtCoord(k,val); }
	
	// Fonte
	void SetFonteC(double valor) { fonteC=valor; }
	double FonteC() const { return fonteC; }
	void SetFonteP(double valor) { fonteP=valor; }
	double FonteP() const { return fonteP; }
	
	// Vizinho
	IntPos NVizinho()  { return vizinho.size(); }
	Celula* CelVizinho(IntPos val) { return vizinho[val].cel; }
	Celula* CelVizOposta(IntPos val) { return vizinho[VizOposto(val)].cel; }
	Face* FaceVizinho(IntPos val) { return vizinho[val].face; }
	Face* FaceVizOposta(IntPos val) { return vizinho[VizOposto(val)].face; }
	Sentido SentidoVizinho(IntPos val) { return vizinho[val].sentido; }
	void SetCoefTermViz(IntPos val, double coef) { vizinho[val].coefTermViz=coef; }
	double CoefTermViz(IntPos val) { return vizinho[val].coefTermViz; }
	void SetCoefTermCel(double coef) { coefTermCel=coef; }
	double CoefTermCel() { return coefTermCel; }
	void SomaCoefTermCel(double coef) { coefTermCel+=coef; }

	Face* FaceSentido(Sentido sent);
        Celula* CelVizSentido(Sentido sent);

	//vector<ElVizinho> VetVizinho() { return vizinho; }
	//Propriedade Termica
	//void SetPropTerm(PropTerm *valor) { propTerm=valor; }
	double CondTerm() { return compTec->CondTerm(); }
	double GerCalor() 
		{ return compTec->GerCalorEsp(); }

	double VazSangueEsp() { return compTec->VazSangueEsp(); }
	double Densidade() { return compTec->Densidade(); }
	double CalorEsp() { return compTec->CalorEsp(); }

	IntPos NumCompCel() { return compTec->NumComp(); }

	TipoTecido NomeTecido() { return compTec->NomeTecido(); }

	//Gases
	/*void SetPressao(Gas<double> valor) { pressao=valor; }
	void SetPressaoO2(double valor) { pressao.O2=valor; }
	void SetPressaoCO2(double valor) { pressao.CO2=valor; }
	Gas<double> Pressao() const { return pressao; }
	void SetConcentracaoO2(double valor) { concentracao.O2=valor; }
	void SetConcentracaoCO2(double valor) { concentracao.CO2=valor; }
	Gas<double> Concentracao() const { return concentracao; }*/

// PERGUNTAS

	bool EhCentro();

// OPERAÇÕES
	
	void NovoVizinho(Celula *vCel, Face *vFace, Sentido vSent);

private:

// ATRIBUTOS
	
	/// Ponto central da celula
	Ponto<double> centro;

	/// Volume
	double volume;
		
	/// Temperatura
	double temperaturaAnt;
	double temperatura;

	/// Tamanho da celula
	Ponto<double> tamanho;

	/// Conjunto de informacoes sobre um vizinho
	struct ElVizinho {
		Celula *cel;
		Face *face;
		Sentido sentido;
		double coefTermViz;
	};

	/// Vetor com as informacoes dos vizinhos
	vector<ElVizinho> vizinho;

	/// Endereco de conjunto das propriedades termicas
	CompTecido *compTec;

	/// Fontes
	double fonteC, fonteP;

	/// Coeficiente termico da face (A_P)
	double coefTermCel;

	/// Pressao e concentracao dos gases
	//Gas<double> pressao, concentracao;

// OPERAÇÕES
	
	/** Calcula o volume de uma celula. 
	 *
	 * \param sistCoord sistema de coordenada
	 */
	void CalcVolume(IntPos nDim, SistCoord sistCoord);
	
	Sentido SentidoOposto(Sentido sent);

	IntPos VizOposto(IntPos val);

	double CorrMetabolismo(double metabolismo);
};

#endif
